@@ -6,8 +6,11 @@ use once_cell::sync::Lazy;
 use futures_util::future::{Either, err};
 use futures_util::stream::{StreamExt, once, iter};
 
-use fluvio_spu_schema::server::stream_fetch::{DefaultStreamFetchRequest, DefaultStreamFetchResponse};
-use dataplane::Isolation;
+use fluvio_spu_schema::server::stream_fetch::{
+    DefaultStreamFetchRequest, DefaultStreamFetchResponse, SmartStreamPayload, SmartStreamWasm,
+    SmartStreamKind, WASM_MODULE_V2_API,
+};
+use dataplane::{Isolation, SmartStreamError};
 use dataplane::ReplicaKey;
 use dataplane::ErrorCode;
 use dataplane::fetch::DefaultFetchRequest;
@@ -22,6 +25,7 @@ use fluvio_types::event::offsets::OffsetPublisher;
 use crate::FluvioError;
 use crate::offset::Offset;
 use crate::spu::SpuPool;
+use derive_builder::Builder;
 
 /// An interface for consuming events from a particular partition
 ///
@@ -109,9 +113,8 @@ impl PartitionConsumer {
         &self,
         offset: Offset,
     ) -> Result<FetchablePartitionResponse<RecordSet>, FluvioError> {
-        let records = self
-            .fetch_with_config(offset, ConsumerConfig::default())
-            .await?;
+        let config = ConsumerConfig::builder().build()?;
+        let records = self.fetch_with_config(offset, config).await?;
         Ok(records)
     }
 
@@ -133,8 +136,9 @@ impl PartitionConsumer {
     /// # use fluvio::{PartitionConsumer, FluvioError, Offset, ConsumerConfig};
     /// # async fn example(consumer: &PartitionConsumer) -> Result<(), FluvioError> {
     /// // Use custom fetching configurations
-    /// let fetch_config = ConsumerConfig::default()
-    ///     .with_max_bytes(1000);
+    /// let fetch_config = ConsumerConfig::builder()
+    ///     .max_bytes(1000)
+    ///     .build()?;
     ///
     /// let response = consumer.fetch_with_config(Offset::beginning(), fetch_config).await?;
     /// for batch in response.records.batches {
@@ -247,9 +251,8 @@ impl PartitionConsumer {
         &self,
         offset: Offset,
     ) -> Result<impl Stream<Item = Result<Record, FluvioError>>, FluvioError> {
-        let stream = self
-            .stream_with_config(offset, ConsumerConfig::default())
-            .await?;
+        let config = ConsumerConfig::builder().build()?;
+        let stream = self.stream_with_config(offset, config).await?;
 
         Ok(stream)
     }
@@ -277,8 +280,9 @@ impl PartitionConsumer {
     /// # async fn example(consumer: &PartitionConsumer) -> Result<(), FluvioError> {
     /// use futures::StreamExt;
     /// // Use a custom max_bytes value in the config
-    /// let fetch_config = ConsumerConfig::default()
-    ///     .with_max_bytes(1000);
+    /// let fetch_config = ConsumerConfig::builder()
+    ///     .max_bytes(1000)
+    ///     .build()?;
     /// let mut stream = consumer.stream_with_config(Offset::beginning(), fetch_config).await?;
     /// while let Some(Ok(record)) = stream.next().await {
     ///     let key: Option<String> = record.key().map(|key| String::from_utf8_lossy(key).to_string());
@@ -329,8 +333,9 @@ impl PartitionConsumer {
     /// # async fn example(consumer: &PartitionConsumer) -> Result<(), FluvioError> {
     /// use futures::StreamExt;
     /// // Use a custom max_bytes value in the config
-    /// let fetch_config = ConsumerConfig::default()
-    ///     .with_max_bytes(1000);
+    /// let fetch_config = ConsumerConfig::builder()
+    ///     .max_bytes(1000)
+    ///     .build()?;
     /// let mut stream = consumer.stream_batches_with_config(Offset::beginning(), fetch_config).await?;
     /// while let Some(Ok(batch)) = stream.next().await {
     ///     for record in batch.records() {
@@ -350,21 +355,31 @@ impl PartitionConsumer {
     ) -> Result<impl Stream<Item = Result<Batch, FluvioError>>, FluvioError> {
         let stream = self.request_stream(offset, config).await?;
         let flattened = stream.flat_map(|batch_result: Result<DefaultStreamFetchResponse, _>| {
-            let response: DefaultStreamFetchResponse = match batch_result {
-                // If error code is None, continue
-                Ok(response) if response.partition.error_code == ErrorCode::None => response,
-                // If error code is anything else, wrap it in an error
-                Ok(response) => {
-                    let code = response.partition.error_code;
-                    return Either::Right(once(err(FluvioError::ApiError(
-                        fluvio_sc_schema::ApiError::Code(code, None),
-                    ))));
-                }
+            let response = match batch_result {
+                Ok(response) => response,
                 Err(e) => return Either::Right(once(err(e))),
             };
 
+            // If we ever get an error_code AND batches of records, we want to first send
+            // the records down the consumer stream, THEN an Err with the error inside.
+            // This way the consumer always gets to read all records that were properly
+            // processed before hitting an error, so that the error does not obscure those records.
             let batches = response.partition.records.batches.into_iter().map(Ok);
-            Either::Left(iter(batches))
+            let error = {
+                let code = response.partition.error_code;
+                match code {
+                    ErrorCode::None => None,
+                    ErrorCode::SmartStreamError(SmartStreamError::Runtime(error)) => {
+                        Some(Err(FluvioError::SmartStreamRuntime(error)))
+                    }
+                    _ => Some(Err(FluvioError::AdminApi(
+                        fluvio_sc_schema::ApiError::Code(code, None),
+                    ))),
+                }
+            };
+
+            let items = batches.chain(error.into_iter());
+            Either::Left(iter(items))
         });
 
         Ok(flattened)
@@ -410,12 +425,21 @@ impl PartitionConsumer {
             .lookup_version(DefaultStreamFetchRequest::API_KEY)
             .unwrap_or((WASM_MODULE_API - 1) as i16);
 
-        if !config.wasm_module.is_empty() {
-            if stream_fetch_version >= WASM_MODULE_API as i16 {
-                stream_request.wasm_module = config.wasm_module;
-            } else {
+        if let Some(module) = config.wasm_module {
+            if stream_fetch_version < WASM_MODULE_API as i16 {
                 return Err(FluvioError::Other("SPU does not support WASM".to_owned()));
             }
+
+            if stream_fetch_version < WASM_MODULE_V2_API as i16 {
+                // SmartStream V1
+                debug!("Using WASM V1 API");
+                let SmartStreamWasm::Raw(wasm) = module.wasm;
+                stream_request.wasm_module = wasm;
+            } else {
+                // SmartStream V2
+                debug!("Using WASM V2 API");
+                stream_request.wasm_payload = Some(module);
+            }
         }
 
         let mut stream = self
@@ -452,14 +476,14 @@ impl PartitionConsumer {
                             session_id = stream_id,
                             "sending back offset to spu"
                         );
-                        let response = serial_socket
-                            .send_receive(UpdateOffsetsRequest {
-                                offsets: vec![OffsetUpdate {
-                                    offset: fetch_last_value,
-                                    session_id: stream_id,
-                                }],
-                            })
-                            .await;
+                        let request = UpdateOffsetsRequest {
+                            offsets: vec![OffsetUpdate {
+                                offset: fetch_last_value,
+                                session_id: stream_id,
+                            }],
+                        };
+                        debug!(?request, "Sending offset update request:");
+                        let response = serial_socket.send_receive(request).await;
                         if let Err(err) = response {
                             error!("error sending offset: {:#?}", err);
                             break;
@@ -565,33 +589,46 @@ static MAX_FETCH_BYTES: Lazy<i32> = Lazy::new(|| {
 });
 
 /// Configures the behavior of consumer fetching and streaming
-#[derive(Debug)]
+#[derive(Debug, Builder)]
+#[builder(build_fn(private, name = "build_impl"))]
 pub struct ConsumerConfig {
+    #[builder(default = "*MAX_FETCH_BYTES")]
     pub(crate) max_bytes: i32,
+    #[builder(default)]
     pub(crate) isolation: Isolation,
-    wasm_module: Vec<u8>,
-}
-
-impl Default for ConsumerConfig {
-    fn default() -> Self {
-        Self {
-            max_bytes: *MAX_FETCH_BYTES,
-            isolation: Isolation::default(),
-            wasm_module: vec![],
-        }
-    }
+    #[builder(private, default, setter(into, strip_option))]
+    pub(crate) wasm_module: Option<SmartStreamPayload>,
 }
 
 impl ConsumerConfig {
-    /// Maximum number of bytes to be fetched at a time.
-    pub fn with_max_bytes(mut self, max_bytes: i32) -> Self {
-        self.max_bytes = max_bytes;
+    pub fn builder() -> ConsumerConfigBuilder {
+        ConsumerConfigBuilder::default()
+    }
+}
+
+impl ConsumerConfigBuilder {
+    pub fn build(&self) -> Result<ConsumerConfig, FluvioError> {
+        let config = self.build_impl().map_err(|e| {
+            FluvioError::ConsumerConfig(format!("Missing required config option: {}", e))
+        })?;
+        Ok(config)
+    }
+
+    /// Adds a SmartStream filter to this ConsumerConfig
+    pub fn wasm_filter<T: Into<Vec<u8>>>(&mut self, filter: T) -> &mut Self {
+        self.wasm_module(SmartStreamPayload {
+            wasm: SmartStreamWasm::Raw(filter.into()),
+            kind: SmartStreamKind::Filter,
+        });
         self
     }
 
-    /// set wasm filter
-    pub fn with_wasm_filter(mut self, bytes: Vec<u8>) -> Self {
-        self.wasm_module = bytes;
+    /// Adds a SmartStream map to this ConsumerConfig
+    pub fn wasm_map<T: Into<Vec<u8>>>(&mut self, map: T) -> &mut Self {
+        self.wasm_module(SmartStreamPayload {
+            wasm: SmartStreamWasm::Raw(map.into()),
+            kind: SmartStreamKind::Map,
+        });
         self
     }
 }
@@ -631,3 +668,13 @@ impl AsRef<[u8]> for Record {
         self.value()
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_consumer_config_default() {
+        let _config = ConsumerConfig::builder().build().unwrap();
+    }
+}
